/*
 * Copyright 2013 The Polymer Authors. All rights reserved.
 * Use of this source code is governed by a BSD-style
 * license that can be found in the LICENSE file.
 */

(function() {

	function reflect(element, name, meta) {
		return {
			obj: element,
			name: name,
			value: element[name],
			meta: meta && meta[name]
		};
	}

	function reflectProperty(element, name, meta) {
		try {
			if (name[0] === '_' || name[name.length-1] === '_') {
				return;
			}
			var v = element[name];
			if (((v !== null
					&& v !== undefined
					&& typeof v !== 'function'
						&& typeof v !== 'object')
						|| Bindings.getBinding(element, name))
						//&& element.propertyIsEnumerable(k)
						&& !reflectProperty.blacklist[name]) {
				var prop = reflect(element, name, meta);
			}
		} catch(x) {
			// squelch
		}
		return prop;
	}

	reflectProperty.blacklist = {
			EVENT_PREFIX: 1,
			DELEGATES: 1,
			PUBLISHED: 1,
			INSTANCE_ATTRIBUTES: 1,
			PolymerBase: 1,
			STYLE_SCOPE_ATTRIBUTE: 1
	};

	function reflectEvent(element, name, meta) {
		try {
			if (name[0] === '_' || name[name.length-1] === '_') {
				return;
			}
			var v = element[name];
			if (((v !== null
					&& v !== undefined
					&& typeof v !== 'function'
						&& typeof v !== 'object')
						|| Bindings.getBinding(element, name))
						//&& element.propertyIsEnumerable(k)
						&& !reflectEvent.blacklist[name]) {
				var prop = reflect(element, name, meta);
			}
		} catch(x) {
			// squelch
		}
		return prop;
	}

	reflectEvent.blacklist = {
			DELEGATES: 1,
			PUBLISHED: 1,
			INSTANCE_ATTRIBUTES: 1,
			PolymerBase: 1,
			STYLE_SCOPE_ATTRIBUTE: 1
	};

	if (!window.designWindow) {
		designWindow = window;
	}

	var defaultMeta = {
			id: { order: 1, title: 'Widget ID' },
			className: { kind: 'classname-editor', order: 1000, title: 'CSS class name' }
	}

	var defaultOrder = {
			titleText: 3,
			value: 5,
			label: 7,
			name: 9,
			checked: 10,
			pressed: 12,
			minValue: 14,
			maxValue: 15,
			decimalPlaces: 20,
			titleLabel: 25,
			labels: 30,
			visible: 100,
			nonvisible: 110,
			disabled: 112,
			tooltip: 113,
			textContent: 120
	};

	function compareStr(str1, str2) {
		return (str1 < str2 ? -1 : (str1 > str2 ? 1 : 0));
	}

	function compareOrder(a, b)
	{
		var orderA = Number(((a.meta && a.meta.order) ? a.meta.order : (defaultOrder[a.name] || 50)));
		var orderB = Number(((b.meta && b.meta.order) ? b.meta.order : (defaultOrder[b.name] || 50)));
		return (orderA == orderB ? compareStr(a.name, b.name) : (orderA < orderB ? -1 : 1));
	}

	function copyProperties(fromElement, toObj) {
//		derived from polymer-micro._mixinBehavior:
		var names = Object.getOwnPropertyNames(fromElement);
		for (var i = 0, name; i < names.length && (name = names[i]); i++) {
			if (!Polymer.Base._behaviorProperties[name] && !toObj.hasOwnProperty(name)) {
				Polymer.Base.copyOwnProperty(name, fromElement, toObj);
			}
		}
		return toObj;
	};

	function reflectProperties(element, includeDefaultMeta, excludeHidden) {
		var props = [];
		if (element) {
			var meta = element._designMeta && element._designMeta.metaProperties || {};
			if (includeDefaultMeta) {
				meta = copyProperties(defaultMeta, meta);
			}

			var found = [];
			meta && Object.keys(meta).forEach(function(n) {
				if (!found.includes(n)) {
					if (excludeHidden && !meta[n].hidden) {
						found.push(n);
					} else if (!excludeHidden) {
						found.push(n);
					}
				}

			});

			found.forEach(function(k) {
				var v = element[k];
				if (v === null || (typeof v !== 'function')) {
					props.push(reflect(element, k, meta));
				}
			});
		}
		props = props.sort(compareOrder);

		return props;
	}


	function reflectEvents(element) {
		var props = [];
		if (element) {
			var found = {};
			var meta = element._designMeta && element._designMeta.properties;
			meta = Platform.mixin({}, defaultMeta, meta);

			var hep = designWindow.HTMLElement.prototype;
			var hiep = designWindow.HTMLInputElement.prototype;
			// TODO(sjmiles): beware alternate-window `p` that will != window.HTMLElement.prototype
			var p = element.__proto__;
			while (p && !p.hasOwnProperty('PolymerBase') && p != hep && p != hiep) {
				var k = Object.keys(p);
				k.forEach(function(k) {
					if (found[k]) {
						return;
					}
					var prop = reflectEvent(element, k, meta);
					if (prop) {
						props.push(prop);
						found[k] = true;
					}
				});
				p = p.__proto__;
			}
			//
			var more = [];
			if (!Polymer.dom(element).firstElementChild) {
				// pchuong: hide textContent from the user
				//more.push('textContent');
			}
			var whitelist = {};
			//
			meta && Object.keys(meta).forEach(function(n) {
				if (!found[n] && more.indexOf(n) === -1) {
					more.push(n);
					whitelist[n] = true;
				}
			});
			//
			more.forEach(function(k) {
				var v = element[k];
				if ((typeof v !== 'function' && typeof v !== 'object') || whitelist[k]) {
					props.push(reflect(element, k, meta));
				}
			});
		}
		return props;
	}

	function reflectObject(obj, meta) {
		var props = [];
		if (obj && meta) {
			Object.keys(meta).forEach(function(name) {
				props.push(reflect(obj, name, meta));
			});
		}
		return props;
	}

	function reflectObjectProperties(obj) {
		var props = [];

		if (obj) {
			Object.keys(obj).forEach(function(name) {
				var p = reflectProperty(obj, name, {});
				if (p) {
					props.push(p);
				}
			});
		}
		return props;
	}

	function reflectStyles(element, meta) {
		if (element) {
			var style = element.__styleRule ? element.__styleRule.style : element.style;
			return reflectObject(style, meta);
		}
	}

	function reflectObjectEvents(obj) {
		var events = [];
		if (obj) {
			Object.keys(obj).forEach(function(name) {
				var e = reflectEvent(obj, name, {});
				if (e) {
					events.push(e);
					console.log("Found an event: e="+ e.toString());
				}
			});
		}
	}

	function getFilteredProperties(element, filter, excludeHidden)
	{
		var props = reflectProperties(element, true, excludeHidden);
		if (filter)
		{
			var results = [];

			for(var i = 0; i < props.length; i++)
			{
				var prop = props[i];
				if (prop.meta && prop.meta[filter])
				{
					results.push(prop);
				}
			}
			return results;
		}
		else
		{
			return props;
		}
	}

	window.Reflection = {
			properties: reflectProperties,
			styles: reflectStyles,
			events: reflectEvents,
			object: reflectObject,
			objectProperties: reflectObjectProperties,
			objectEvents: reflectObjectEvents,
			filteredProperties: getFilteredProperties
	};

})();
