/*
Copyright (c) 2018, Texas Instruments Incorporated
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

*   Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
*   Neither the name of Texas Instruments Incorporated nor the names of
its contributors may be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
/*jslint node: true */
"use strict";
// Copy this file into the <designer root folder>/win32/TICloudAgentHostApp/src/modules folder (for desktop designer) or
// into the c:/Users/<userid>/ticloudagent/src/modules folder (for cloud-based designer) in order to run this demo
const Q = require("q");
const logger = require("../logger");
const echoObj = { msDelay: 100, timerHdls: [] };
class Echo {
    constructor(_triggerEvent) {
        this._triggerEvent = _triggerEvent;
    }
    open(msDelay, forceFail) {
        const deferred = Q.defer();
        let argMsDelay = msDelay;
        if (isNaN(argMsDelay)) {
            argMsDelay = 1000;
        }
        echoObj.msDelay = argMsDelay;
        logger.info(`Echo.open called! msDelay=${msDelay}, forceFail=${forceFail}`);
        if (!forceFail) {
            deferred.resolve();
        }
        else {
            const err = "open failed.";
            logger.info(err);
            deferred.reject({
                message: err,
            });
        }
        return deferred.promise;
    }
    write(strData, forceFail) {
        const deferred = Q.defer();
        if (!forceFail) {
            const timerHdl = setTimeout(() => {
                this._onDataCallback(strData);
            }, echoObj.msDelay);
            echoObj.timerHdls.push(timerHdl);
            deferred.resolve();
        }
        else {
            this._onErrorCallback("forceFail = true, data=" + strData);
            deferred.reject("writeData failed - see error callback for details.");
        }
        return deferred.promise;
    }
    close() {
        logger.info(`Echo.closePort`);
        while (echoObj.timerHdls.length > 0) {
            clearTimeout(echoObj.timerHdls[0]);
            echoObj.timerHdls.shift();
        }
        const deferred = Q.defer();
        deferred.resolve();
        return deferred.promise;
    }
    _onDataCallback(strData) {
        logger.info(`Echo._onDataCallback(${strData})`);
        echoObj.timerHdls.shift();
        const dataJSON = '{"data":"' + strData + '"}';
        const dataPOD = JSON.parse(dataJSON);
        this._triggerEvent("echo", {
            buffer: dataPOD.data,
        });
    }
    _onErrorCallback(error) {
        logger.info(`Echo._onErrorCallback( error = ${error.toString()}`);
        if (this._triggerEvent) {
            this._triggerEvent("dataError", {
                error,
            });
        }
    }
}
exports.Echo = Echo;
exports.name = "Echo"; // this is the string that the service needs to pass into the agent.getSubModule API
function instance(triggerEvent) {
    return {
        commands: new Echo(triggerEvent),
    };
}
exports.instance = instance;
